package libdrm

  1. Overview
  2. Docs

Module Drm.KmsSource

Kernel mode-setting.

This API is used to configure physical monitors (screen resolution, framebuffers, etc).

See the Linux Kernel Mode Setting documentation for more information.

type uint32 := Unsigned.UInt32.t
Sourcemodule type BITSET = sig ... end
Sourcemodule Rect : sig ... end
Sourcemodule Blob : sig ... end

An untyped string of bytes.

Sourcemodule Mode_info : sig ... end

Screen mode resolution and timing information.

Sourcemodule Sub_pixel : sig ... end

How pixels are physically arranged on a particular monitor.

Sourcemodule Property : sig ... end

A property can be used to read or write extra settings on an object (resource).

Sourcemodule Properties : sig ... end

Drivers can add extra properties to objects, discoverable at runtime.

Sourcemodule Connector : sig ... end

A physical connector used to attach a monitor.

Sourcemodule Fb : sig ... end

A framebuffer manages the inputs to a Crtc.

Sourcemodule Crtc : sig ... end

A CRT Controller.

Sourcemodule Plane : sig ... end

A plane wraps a Buffer with some extra metadata.

Sourcemodule Encoder : sig ... end

Encoders reads pixel data from a Crtc and output it in a suitable format for a Connector.

Sourcemodule Resources : sig ... end

The set of resources provided by the hardware (excluding planes; see Plane.list).

Sourcemodule Atomic_req : sig ... end

Atomic requests.

Sourcemodule Lease : sig ... end

Sharing access with other processes.