package libdrm

  1. Overview
  2. Docs

Module Drm.ModifierSource

Format modifiers.

Sourcetype t = private int64
Sourceval of_int64 : int64 -> t
Sourceval of_uint64 : Unsigned.UInt64.t -> t
Sourceval to_uint64 : t -> Unsigned.UInt64.t
Sourceval pp : t Fmt.t
Sourceval reserved : t
Sourceval linear : t

Just plain linear layout. Note that this is different from not specifying any modifier, which tells the driver to also take driver-internal information into account and so might actually result in a tiled framebuffer.